@@ -627,6 +627,12 @@ else {
 		    )];
 }
 
+if ($define{'PERL_MALLOC_WRAP'}) {
+    emit_symbols [qw(
+		    PL_memory_wrap
+		    )];
+}
+
 unless ($define{'USE_5005THREADS'} || $define{'USE_ITHREADS'}) {
     skip_symbols [qw(
 		    PL_thr_key
